module aam;

private {
	import win32.ansi.windows;
	import nsmsgs.consts, nsmsgs.types;
	import wstring;
	import std.string, std.utf;
}

extern (C) {
	void gc_init();
	void gc_term();
	void _minit();
	void _moduleCtor();
	void _moduleUnitTests();
}

private {
	const char[][] PluginInfo = [
		NSM_API_VERSION,                     // API Version
		"AddIn/aam",                         // Module Name
		"An Answering Machine on Regnessem", // Plugin Title
		"An Answering Machine on Regnessem", // Description
		"Moon",                              // Author
		"Copyright (c) 2004 Moon",           // Copyright
		"0.0.4",                             // Version
	];
	const wchar[][] MainMenu = [
		"留守番機能 ON",
		"留守番機能 OFF",
	];
	static bool MenuStatus;
	TNsmPluginInitInfo PluginInitInfo;
}

private const char[] NMS_ADDIN_AAM_UISERVICE_MAIN_SWITCH = "AddIn/aam/UIService/Main/Switch";

extern (Windows)
BOOL DllMain(HINSTANCE hInstance, ULONG ulReason, LPVOID pvReserved)
{
	switch (ulReason) {
	case DLL_PROCESS_ATTACH:
		gc_init();          // init GC
		_minit();           // init module list
		_moduleCtor();      // run  module constructor
		_moduleUnitTests(); // run  unit test
		break;
	case DLL_PROCESS_DETACH:
		gc_term();
		break;
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
		// multi-thread is not supported
		return FALSE;
	}
	return TRUE;
}

// DLL Export Functions

extern (Windows)
int GetPluginInfo(int nInfoNo, LPTSTR lpBuffer, int nSize)
{
	if (nInfoNo < 0 || nInfoNo > PluginInfo.length) {
		// Unknown Information Number
		return 0;
	} else {
		lstrcpyn(lpBuffer, toStringz(PluginInfo[nInfoNo]), nSize);
		return lstrlen(lpBuffer);
	}
}

extern (Windows)
int Initialize(PNsmPluginInitInfo lpInitInfo)
{
	PluginInitInfo = *lpInitInfo;
	// Create Plugin Services
	PluginInitInfo.CreateService(toStringz(NMS_ADDIN_AAM_UISERVICE_MAIN_SWITCH), &DoSwitch);
	// Hook Events
	PluginInitInfo.HookEvent(toStringz(NME_SYSTEM_SESSION_RECEIVEMESSAGE), &OnReceiveMessage);
	return 0;
}

extern (Windows)
int Terminate()
{
	// Unhook Events
	PluginInitInfo.UnhookEvent(toStringz(NME_SYSTEM_SESSION_RECEIVEMESSAGE), &OnReceiveMessage);
	return 0;
}

// Services

extern (Windows)
int DoSwitch(WPARAM wParam, LPARAM lParam)
{
	HNsmConnection    hConnection = cast(HNsmConnection)    wParam;
	PNsmUIServiceInfo lpUISvcInfo = cast(PNsmUIServiceInfo) lParam;
	
	switch (lpUISvcInfo.nInfoKey) {
	case NMUI_GETCAPTION:
		lstrcpynW(
			cast(wchar*) lpUISvcInfo.lpInfo.lpBuffer,
			toWStringz(MainMenu[(MenuStatus ? 1 : 0)]),
			lpUISvcInfo.lpInfo.nBufferSize);
		break;
	case NMUI_ONCLICK:
		MenuStatus = !MenuStatus;
		break;
	default:
		// Unknown Type
		return 0;
	}
	return 1;
}

// Event
extern (Windows)
int OnReceiveMessage(WPARAM wParam, LPARAM lParam)
{
	HNsmSession  hSession  = cast(HNsmSession)  wParam;
	PMessageInfo lpMesInfo = cast(PMessageInfo) lParam;
	
	if (MenuStatus) {
		HNsmService hService = PluginInitInfo.GetService(toStringz(NMS_SYSTEM_SESSION_SENDMESSAGE));
		if (hService) {
			TTextAttributeInfo textattr;
			textattr.cbSize     = TTextAttributeInfo.sizeof;
			textattr.lpFontName = lpMesInfo.lpTextAttribute.lpFontName;
			textattr.nCharSet   = lpMesInfo.lpTextAttribute.nCharSet;
			textattr.nFontSize  = lpMesInfo.lpTextAttribute.nFontSize;
			textattr.nFontColor = lpMesInfo.lpTextAttribute.nFontColor;
			textattr.nBgColor   = lpMesInfo.lpTextAttribute.nBgColor;
			textattr.nStyles    = lpMesInfo.lpTextAttribute.nStyles;
			
			TMessageInfo mesinfo;
			mesinfo.cbSize          = TMessageInfo.sizeof;
			mesinfo.lpFrom          = null;
			mesinfo.lpBody          = toWStringz("ただいま留守にしています。ご用のある方はメッセージをどうぞ。");
			mesinfo.lpTextAttribute = &textattr;
			
			PluginInitInfo.CallService(hService, cast(WPARAM) hSession, cast(LPARAM) &mesinfo);
		}
	}
	
	return 0;
}
